params = {
    "tool_usbcam_wrist": {
        'py_class_name': 'ToolUSBCamWrist',
        'py_module_name': 'stretch_tool_share.usbcam_wrist_v1.tool',
        'use_group_sync_read': 1,
        'baud': 57600,
        'dxl_latency_timer': 64,
        'retry_on_comm_failure': 1,
        'verbose':0,
        'stow': {'wrist_yaw': 3.4},
        'devices': {
            'wrist_yaw': {
                'py_class_name': 'OffsetWristYaw',
                'py_module_name': 'stretch_tool_share.usbcam_wrist_v1.offset_wrist_yaw',
                'ros_py_class_name': 'OffsetWristYawCommandGroup',
                'ros_py_module_name': 'stretch_tool_share.usbcam_wrist_v1.command_groups'
            }
        }
    }
}



from hello_helpers.simple_command_group import SimpleCommandGroup


class ReactorPitchCommandGroup(SimpleCommandGroup):
    def __init__(self, range_rad, robot=None):
        if (range_rad is None and robot is not None and robot.end_of_arm is not None
            and robot.end_of_arm.is_tool_present('ReactorWrist')
            and robot.end_of_arm.joints['reactor_wrist'].motors.get('reactor_pitch') is not None):
            range_ticks = robot.end_of_arm.motors['reactor_wrist'].params['range_t']
            range_rad = (robot.end_of_arm.motors['reactor_wrist'].ticks_to_world_rad(range_ticks[1]),
                         robot.end_of_arm.motors['reactor_wrist'].ticks_to_world_rad(range_ticks[0]))
        SimpleCommandGroup.__init__(self, 'joint_reactor_pitch', range_rad)

    def init_execution(self, robot, robot_status, **kwargs):
        if self.active:
            robot.end_of_arm.joints['reactor_wrist'].move_by('reactor_pitch',
                                     self.update_execution(robot_status)[1],
                                     v_r=self.goal['velocity'],
                                     a_r=self.goal['acceleration'])

    def update_execution(self, robot_status, **kwargs):
        self.error = None
        if self.active:
            self.error = self.goal['position'] - robot_status.end_of_arm.joints['reactor_wrist'].motors['reactor_pitch'].status['pos']
            return self.name, self.error

        return None

    def joint_state(self, robot_status, **kwargs):
        pitch_status = robot_status['end_of_arm']['reactor_wrist']
        return (pitch_status['pos'], pitch_status['vel'], pitch_status['effort'])


class ReactorRollCommandGroup(SimpleCommandGroup):
    def __init__(self, range_rad, robot=None):
        if (range_rad is None and robot is not None and robot.end_of_arm is not None
            and robot.end_of_arm.is_tool_present('ReactorWrist')
            and robot.end_of_arm.joints['reactor_wrist'].motors.get('reactor_roll') is not None):
            range_ticks = robot.end_of_arm.motors['reactor_wrist'].params['range_t']
            range_rad = (robot.end_of_arm.motors['reactor_wrist'].ticks_to_world_rad(range_ticks[1]),
                         robot.end_of_arm.motors['reactor_wrist'].ticks_to_world_rad(range_ticks[0]))
        SimpleCommandGroup.__init__(self, 'joint_reactor_roll', range_rad)

    def init_execution(self, robot, robot_status, **kwargs):
        if self.active:
            robot.end_of_arm.joints['reactor_wrist'].move_by('reactor_roll',
                                     self.update_execution(robot_status)[1],
                                     v_r=self.goal['velocity'],
                                     a_r=self.goal['acceleration'])

    def update_execution(self, robot_status, **kwargs):
        self.error = None
        if self.active:
            self.error = self.goal['position'] - robot_status.end_of_arm.joints['reactor_wrist'].motors['reactor_roll'].status['pos']
            return self.name, self.error

        return None

    def joint_state(self, robot_status, **kwargs):
        pitch_status = robot_status['end_of_arm']['reactor_wrist']
        return (pitch_status['pos'], pitch_status['vel'], pitch_status['effort'])


class ReactorGripperCommandGroup(SimpleCommandGroup):
    def __init__(self, range_rad, robot=None):
        if (range_rad is None and robot is not None and robot.end_of_arm is not None
            and robot.end_of_arm.is_tool_present('ReactorWrist')
            and robot.end_of_arm.joints['reactor_wrist'].motors.get('stretch_gripper') is not None):
            range_ticks = robot.end_of_arm.motors['reactor_wrist'].params['range_t']
            range_rad = (robot.end_of_arm.motors['reactor_wrist'].ticks_to_world_rad(range_ticks[1]),
                         robot.end_of_arm.motors['reactor_wrist'].ticks_to_world_rad(range_ticks[0]))
        SimpleCommandGroup.__init__(self, 'joint_reactor_gripper', range_rad)

    def init_execution(self, robot, robot_status, **kwargs):
        if self.active:
            robot.end_of_arm.joints['reactor_wrist'].move_by('reactor_gripper',
                                     self.update_execution(robot_status)[1],
                                     v_r=self.goal['velocity'],
                                     a_r=self.goal['acceleration'])

    def update_execution(self, robot_status, **kwargs):
        self.error = None
        if self.active:
            self.error = self.goal['position'] - robot_status.end_of_arm.joints['reactor_wrist'].motors['reactor_gripper'].status['pos']
            return self.name, self.error

        return None

    def joint_state(self, robot_status, **kwargs):
        pitch_status = robot_status['end_of_arm']['reactor_wrist']
        return (pitch_status['pos'], pitch_status['vel'], pitch_status['effort'])



#!/usr/bin/env python

import reactorx_wrist_V1.python.reactor_wrist as reactor_wrist
from stretch_body.hello_utils import *
import sys


r=reactor_wrist.ReactorWrist()
r.startup()

v_des=[r.motors['reactor_pitch'].params['motion']['default']['vel'], r.motors['reactor_roll'].params['motion']['default']['vel'],r.motors['reactor_gripper'].params['motion']['default']['vel']]
a_des=[r.motors['reactor_pitch'].params['motion']['default']['accel'], r.motors['reactor_roll'].params['motion']['default']['accel'],r.motors['reactor_gripper'].params['motion']['default']['accel']]


def menu_top():
    print('------ MENU -------')
    print('m: menu')
    print('a: increment pitch 10 deg')
    print('b: decrement pitch 10 deg')
    print('c: increment roll 10 deg')
    print('d: decrement roll 10 deg')
    print('e: increment gripper 10 deg')
    print('f: decrement gripper 10 deg')
    print('-----')
    print('p: pitch go to pos ticks')
    print('r: roll go to pos ticks')
    print('g: gripper go to pos ticks')
    print('-----')
    print('0: stow')
    print('1: tool up 0')
    print('2: tool up 90')
    print('3: tool up -90')
    print('4: tool down 0')
    print('5: tool down 90')
    print('6: tool down -90')
    print('7: gripper close')
    print('8: gripper open')
    print('-----')
    print('x: speed slow')
    print('y: speed default')
    print('z: speed fast')
    print('-------------------')

def step_interaction():
    global v_des, a_des
    menu_top()
    x=sys.stdin.readline()
    r.pull_status()
    if len(x)>1:
        if x[0]=='m':
            menu_top()
        if x[0]=='p':
            p = float(x[1:])
            p = r.motors['reactor_pitch'].ticks_to_world_rad(p)
            r.move_to('reactor_pitch',p,v_des[0], a_des[0])
        if x[0]=='r':
            t = float(x[1:])
            t = r.motors['reactor_roll'].ticks_to_world_rad(t)
            r.move_to('reactor_roll', t, v_des[1], a_des[1])
        if x[0] == 'g':
            g = float(x[1:])
            g = r.motors['reactor_gripper'].ticks_to_world_rad(g)
            r.move_to('reactor_gripper', g, v_des[2], a_des[2])
        if x[0] == 'a':
            r.move_by('reactor_pitch', deg_to_rad(10), v_des[0], a_des[0])
        if x[0] == 'b':
            r.move_by('reactor_pitch', deg_to_rad(-10), v_des[0], a_des[0])
        if x[0] == 'c':
            r.move_by('reactor_roll', deg_to_rad(10), v_des[1], a_des[1])
        if x[0] == 'd':
            r.move_by('reactor_roll', deg_to_rad(-10), v_des[1], a_des[1])
        if x[0] == 'e':
            r.move_by('reactor_gripper', deg_to_rad(10), v_des[2], a_des[2])
        if x[0] == 'f':
            r.move_by('reactor_gripper', deg_to_rad(-10), v_des[2], a_des[2])

        if x[0] == 'x':
            v_des = [r.motors['reactor_pitch'].params['motion']['slow']['vel'],
                     r.motors['reactor_roll'].params['motion']['slow']['vel'],
                     r.motors['reactor_gripper'].params['motion']['slow']['vel']]
            a_des = [r.motors['reactor_pitch'].params['motion']['slow']['accel'],
                     r.motors['reactor_roll'].params['motion']['slow']['accel'],
                     r.motors['reactor_gripper'].params['motion']['slow']['accel']]

        if x[0] == 'y':
            v_des = [r.motors['reactor_pitch'].params['motion']['default']['vel'],
                     r.motors['reactor_roll'].params['motion']['default']['vel'],
                     r.motors['reactor_gripper'].params['motion']['default']['vel']]
            a_des = [r.motors['reactor_pitch'].params['motion']['default']['accel'],
                     r.motors['reactor_roll'].params['motion']['default']['accel'],
                     r.motors['reactor_gripper'].params['motion']['default']['accel']]

        if x[0] == 'z':
            v_des = [r.motors['reactor_pitch'].params['motion']['fast']['vel'],
                     r.motors['reactor_roll'].params['motion']['fast']['vel'],
                     r.motors['reactor_gripper'].params['motion']['fast']['vel']]
            a_des = [r.motors['reactor_pitch'].params['motion']['fast']['accel'],
                     r.motors['reactor_roll'].params['motion']['fast']['accel'],
                     r.motors['reactor_gripper'].params['motion']['fast']['accel']]

        if x[0] == '0':
            r.pose('stow')
        if x[0] == '1':
            r.pose('tool_up_0')
        if x[0] == '2':
            r.pose('tool_up_90')
        if x[0] == '3':
            r.pose('tool_up_-90')
        if x[0] == '4':
            r.pose('tool_down_0')
        if x[0] == '5':
            r.pose('tool_down_90')
        if x[0] == '6':
            r.pose('tool_down_-90')
        if x[0] == '7':
            r.pose('tool_down_-90')
        r.push_command()
    else:
        r.pretty_print()

try:
    while True:
        try:
            step_interaction()
        except (ValueError):
            print('Bad input...')
except (ThreadServiceExit, KeyboardInterrupt):
    r.stop()






from stretch_body.dynamixel_hello_XL430 import DynamixelHelloXL430
from stretch_body.hello_utils import *
from stretch_body.dynamixel_X_chain import DynamixelXChain

"""
To use this class, include in stretch_re1_user_params.yaml:

end_of_arm:
  devices:
    reactor_wrist: {py_class_name: ReactorWrist, py_module_name: reactor_wrist}
    
Ensure reactor_wrist.py is on the PYTHONPATH

Ensure that stretch_re1_tool_params.yaml include entries for:
-- reactor_gripper
-- reactor_pitch
-- reactor_roll

"""
class ReactorWrist(DynamixelXChain):
    def __init__(self):
        DynamixelXChain.__init__(self,'/dev/hello-dynamixel-wrist')
        pitch=DynamixelHelloXL430('reactor_pitch',self)
        self.add_motor(pitch)
        roll=DynamixelHelloXL430('reactor_roll',self)
        self.add_motor(roll)
        gripper = DynamixelHelloXL430('reactor_gripper', self)
        self.add_motor(gripper)
        self.poses = {'stow': [deg_to_rad(-90), deg_to_rad(0)],
                      'tool_ahead_0': [deg_to_rad(0), deg_to_rad(0)],
                      'tool_ahead_90': [deg_to_rad(0), deg_to_rad(90)],
                      'tool_ahead_-90': [deg_to_rad(0), deg_to_rad(-90)]}

    # ###################################################
    def move_to(self, joint, x_r, v_r=None, a_r=None):
        """
        joint: Name of the joint to move ('reactor_pitch' or 'reactor_roll' or 'reactor_gripper')
        x_r: commanded absolute position (radians).
        v_r: velocity for trapezoidal motion profile (rad/s).
        a_r: acceleration for trapezoidal motion profile (rad/s^2)
        """
        self.motors[joint].move_to(x_r,v_r,a_r)

    def move_by(self, joint,  x_r, v_r=None, a_r=None):
        """
        joint: Name of the joint to move ('reactor_pitch' or 'reactor_roll' or 'reactor_gripper')
        x_r: commanded incremental motion (radians).
        v_r: velocity for trapezoidal motion profile (rad/s).
        a_r: acceleration for trapezoidal motion profile (rad/s^2)
        """
        self.motors[joint].move_by(x_r,v_r,a_r)


    def pose(self, p):
        """
        p: Dictionary key to named pose (eg 'stow'). Doesn't do gripper.
        """
        self.move_to('reactor_pitch', self.poses[p][0])
        self.move_to('reactor_roll', self.poses[p][1])


from stretch_body.wrist_yaw import WristYaw
from stretch_body.hello_utils import *


class OffsetWristYaw(WristYaw):
    """
    API to the Stretch RE1 wrist yaw joint with an artificial 90 degree offset
    """
    def __init__(self, chain=None):
        WristYaw.__init__(self, chain)


    def home(self):
        """
        Home to hardstops
        """
        WristYaw.home(self, single_stop=True, move_to_zero=False)
        self.move_to(deg_to_rad(90.0))
        self.push_command()







params = {
    'robot_sentry': {
        'wrist_pitch_overload': 1,
        'wrist_roll_overload': 1},
    "tool_stretch_dex_wrist": {
        'py_class_name': 'ToolStretchDexWrist',
        'py_module_name': 'stretch_tool_share.stretch_dex_wrist_beta.tool',
        'use_group_sync_read': 1,
        'retry_on_comm_failure': 1,
        'baud':115200,
        'dxl_latency_timer': 64,
        'verbose':0,
        'stow': {
            'arm': 0.0,
            'lift': 0.38,
            'stretch_gripper': 0.0,
            'wrist_pitch': -1.57,
            'wrist_roll': 0.0,
            'wrist_yaw': 0.0
        },
        'devices': {
            'stretch_gripper': {
                'py_class_name': 'StretchGripper',
                'py_module_name': 'stretch_body.stretch_gripper',
            },
            'wrist_pitch': {
                'py_class_name': 'WristPitch',
                'py_module_name': 'stretch_tool_share.stretch_dex_wrist_beta.wrist_pitch',
                'ros_py_class_name': 'WristPitchCommandGroup',
                'ros_py_module_name': 'stretch_tool_share.stretch_dex_wrist_beta.command_groups'
            },
            'wrist_roll': {
                'py_class_name': 'WristRoll',
                'py_module_name': 'stretch_tool_share.stretch_dex_wrist_beta.wrist_roll',
                'ros_py_class_name': 'WristRollCommandGroup',
                'ros_py_module_name': 'stretch_tool_share.stretch_dex_wrist_beta.command_groups'
            },
            'wrist_yaw': {
                'py_class_name': 'WristYaw',
                'py_module_name': 'stretch_body.wrist_yaw',
            },
        },
        'collision_models': ['collision_stretch_dex_wrist_to_self','collision_stretch_dex_wrist_to_base']
    },
    "collision_stretch_dex_wrist_to_base": {
        'enabled': 1,
        'py_class_name': 'CollisionStretchDexWristToBase',
        'py_module_name': 'stretch_tool_share.stretch_dex_wrist_beta.collision_model',
        'palm_height':0.12,
        'arm_clear_base':0.086,
        'base_to_floor_offset':0.16,
        'lift_up':0.37
    },
    "collision_stretch_dex_wrist_to_self": {
        'enabled': 1,
        'pitch_up_thresh': 0.1,
        'limit_pitch_up_palm_down': {'pitch': [None, 0.45], 'yaw': [-0.50, 3.14]},
        'limit_pitch_up_palm_side': {'pitch': [None, 0.45], 'yaw': [-0.50, 3.14]},
        'limit_pitch_up_palm_up': {'pitch': [None, 0], 'yaw': [None, 3.14]},
        'py_class_name': 'CollisionStretchDexWristToSelf',
        'py_module_name': 'stretch_tool_share.stretch_dex_wrist_beta.collision_model',
    },
    "wrist_pitch": {
        'flip_encoder_polarity': 1,
        'enable_runstop': 1,
        'gr': 1.0,
        'id': 15,
        'max_voltage_limit': 15,
        'min_voltage_limit': 11,
        'motion': {
            'default': {'accel': 8.0, 'vel': 3.0},
            'fast': {'accel': 10.0, 'vel': 5.0},
            'max': {'accel': 12, 'vel': 8},
            'slow': {'accel': 4.0, 'vel': 1.0}
        },
        'pid': [400, 0, 200],
        'pwm_homing': [0, 0],
        'pwm_limit': 885,
        'range_t': [650, 2048],
        'req_calibration': 0,
        'return_delay_time': 0,
        'stall_backoff': 0.017,
        'stall_max_effort': 10.0,
        'stall_max_time': 1.0,
        'stall_min_vel': 0.1,
        'temperature_limit': 72,
        'usb_name': '/dev/hello-dynamixel-wrist',
        'use_multiturn': 0,
        'zero_t': 1024,
        'baud':115200,
        'retry_on_comm_failure':1,
        'verbose': 0
    },
    "wrist_roll": {
        'flip_encoder_polarity': 0,
        'enable_runstop': 1,
        'gr': 1.0,
        'id': 16,
        'max_voltage_limit': 16,
        'min_voltage_limit': 9,
        'motion': {
            'default': {'accel': 8.0, 'vel': 3.0},
            'fast': {'accel': 10.0, 'vel': 5.0},
            'max': {'accel': 12, 'vel': 8},
            'slow': {'accel': 4.0, 'vel': 1.0}
        },
        'pid': [800, 0, 0],
        'pwm_homing': [0, 0],
        'pwm_limit': 885,
        'range_t': [0, 4095],
        'req_calibration': 0,
        'return_delay_time': 0,
        'stall_backoff': 0.017,
        'stall_max_effort': 10.0,
        'stall_max_time': 1.0,
        'stall_min_vel': 0.1,
        'temperature_limit': 80,
        'usb_name': '/dev/hello-dynamixel-wrist',
        'use_multiturn': 0,
        'zero_t': 2048,
        'baud': 115200,
        'retry_on_comm_failure': 1,
        'verbose': 0
    }
}



from stretch_body.end_of_arm import EndOfArm


class ToolReactorWrist(EndOfArm):
    def __init__(self, name='tool_reactor_wrist'):
        EndOfArm.__init__(self, name)

    def stow(self):
        # Fold in wrist and gripper
        print('--------- Stowing Wrist Yaw ----')
        self.move_to('wrist_yaw', self.params['stow']['wrist_yaw'])
        print('--------- Stowing Reactor ----')
        self.motors['reactor_wrist'].pose('stow')



from stretch_body.end_of_arm import EndOfArm



class ToolStretchDexWristBeta(EndOfArm):
    def __init__(self,name='tool_stretch_dex_wrist'):
        EndOfArm.__init__(self,name)

    def stow(self):
        # Fold in wrist and gripper
        print('--------- Stowing ToolStretchDexWrist ----')
        self.move_to('wrist_pitch', self.params['stow']['wrist_pitch'])
        self.move_to('wrist_roll', self.params['stow']['wrist_roll'])
        self.move_to('wrist_yaw', self.params['stow']['wrist_yaw'])
        self.move_to('stretch_gripper', self.params['stow']['stretch_gripper'])
    def home(self):
        self.motors['stretch_gripper'].home()
        if self.motors['wrist_pitch'].status['pos']>0:
            self.motors['wrist_pitch'].move_to(0)
        self.motors['wrist_yaw'].home()


from hello_helpers.simple_command_group import SimpleCommandGroup


class WristPitchCommandGroup(SimpleCommandGroup):
    def __init__(self, range_rad, robot=None):
        if (range_rad is None and robot is not None and robot.end_of_arm is not None
            and robot.end_of_arm.is_tool_present('WristPitch')):
            range_ticks = robot.end_of_arm.motors['wrist_pitch'].params['range_t']
            range_rad = (robot.end_of_arm.motors['wrist_pitch'].ticks_to_world_rad(range_ticks[1]),
                         robot.end_of_arm.motors['wrist_pitch'].ticks_to_world_rad(range_ticks[0]))
        SimpleCommandGroup.__init__(self, 'joint_wrist_pitch', range_rad)

    def init_execution(self, robot, robot_status, **kwargs):
        if self.active:
            robot.end_of_arm.move_by('wrist_pitch',
                                     self.update_execution(robot_status)[1],
                                     v_r=self.goal['velocity'],
                                     a_r=self.goal['acceleration'])

    def update_execution(self, robot_status, **kwargs):
        self.error = None
        if self.active:
            self.error = self.goal['position'] - robot_status['end_of_arm']['wrist_pitch']['pos']
            return self.name, self.error

        return None

    def joint_state(self, robot_status, **kwargs):
        pitch_status = robot_status['end_of_arm']['wrist_pitch']
        return (pitch_status['pos'], pitch_status['vel'], pitch_status['effort'])


class WristRollCommandGroup(SimpleCommandGroup):
    def __init__(self, range_rad, robot=None):
        if (range_rad is None and robot is not None and robot.end_of_arm is not None
            and robot.end_of_arm.is_tool_present('WristRoll')):
            range_ticks = robot.end_of_arm.motors['wrist_roll'].params['range_t']
            range_rad = (robot.end_of_arm.motors['wrist_roll'].ticks_to_world_rad(range_ticks[0]),
                         robot.end_of_arm.motors['wrist_roll'].ticks_to_world_rad(range_ticks[1]))
        SimpleCommandGroup.__init__(self, 'joint_wrist_roll', range_rad)

    def init_execution(self, robot, robot_status, **kwargs):
        if self.active:
            robot.end_of_arm.move_by('wrist_roll',
                                     self.update_execution(robot_status)[1],
                                     v_r=self.goal['velocity'],
                                     a_r=self.goal['acceleration'])

    def update_execution(self, robot_status, **kwargs):
        self.error = None
        if self.active:
            self.error = self.goal['position'] - robot_status['end_of_arm']['wrist_roll']['pos']
            return self.name, self.error

        return None

    def joint_state(self, robot_status, **kwargs):
        roll_status = robot_status['end_of_arm']['wrist_roll']
        return (roll_status['pos'], roll_status['vel'], roll_status['effort'])
